require 'open-uri' # open(url).read

class OsmTrailsController < ApplicationController
  before_action :set_osm_trail, only: [:show, :edit, :update, :destroy, :reimport, :reverse]
  before_action :ensure_admin!
  helper 'osm'

  # GET /osm_trails
  # GET /osm_trails.json
  def index
    @osm_trails = OsmTrail.all    
    
    # filter
    @osm_trails = @osm_trails.search(params[:search]) unless params[:search].blank?
    
    # sort
    @osm_trails = case params[:order]
      when 'pilsen'   then @osm_trails.sort_by{|ot| Map.distance(ot.gpx.first_point, [49.7477415, 13.3775249]) || 9999 }
      when 'priority' then @osm_trails.joins(:note).order('coalesce(osm_trail_notes.priority, 999999),barva')
      when 'length'   then @osm_trails.order('delka DESC')
      else  @osm_trails.order('coalesce(kctref, 999999)')
    end
    
    # paginate
    @osm_trails = @osm_trails.page(params[:page]) unless @osm_trails.is_a? Array
  end

  # GET /osm_trails/1
  # GET /osm_trails/1.json
  def show
    raise "This osm trail has not been found" if  @osm_trail.nil?
#     raise @osm_trail.gpx.flat.first.inspect
  end

  # GET /osm_trails/new
  def new
    @osm_trail = OsmTrail.new
  end

  # GET /osm_trails/1/edit
  def edit
  end

  # POST /osm_trails
  # POST /osm_trails.json
  def create
    @osm_trail = OsmTrail.new(osm_trail_params)

    respond_to do |format|
      if @osm_trail.save
        format.html { redirect_to @osm_trail, notice: 'Osm trail was successfully created.' }
        format.json { render action: 'show', status: :created, location: @osm_trail }
      else
        format.html { render action: 'new' }
        format.json { render json: @osm_trail.errors, status: :unprocessable_entity }
      end
    end
  end

  # PATCH/PUT /osm_trails/1
  # PATCH/PUT /osm_trails/1.json
  def update
    if params['commit']=='Reverse'
      @osm_trail.reverse!
      @osm_trail.save
    end
    
    respond_to do |format|
      if @osm_trail.update(osm_trail_params)
        @osm_trail.save
        format.html { redirect_to action: 'edit', notice: 'Osm trail was successfully updated.' }
        format.json { head :no_content }
      else
        format.html { render action: 'edit' }
        format.json { render json: @osm_trail.errors, status: :unprocessable_entity }
      end
    end
  end

  # PATCH /osm_trails/1/reverse
  def reverse
    @osm_trail.reverse!
    @osm_trail.save
    redirect_to :back
  end
    
  # reimports trail from OSM
  # PATCH /osm_trails/1/reimport
  def reimport
    orig_osmid = if @osm_trail.nil? and params[:id].match /osmid-(\d*)/ 
#       raise 'Chtel jsem vedet zda to sem nekdy dojde - chci to predelat do modelu'
      $1
    else
      @osm_trail.andand.osmid
    end
    orig_reversed = @osm_trail.andand.reversed?
    url = "http://overpass-api.de/api/interpreter?data=[out%3Ajson]%3B%28relation%28#{orig_osmid}%29%3B%29%3B%3E%3E%3Bout%20body%3B%0A"
    OsmTools.import_osm_data_from_json_string(open(url).read)
    
    @reimported_osm_trail = OsmTrail.find_by_osmid(orig_osmid)
    @reimported_osm_trail.reverse! if orig_reversed==1
    flash.notice = 'Trail successfully reiported from OSM'
    if !request.env["HTTP_REFERER"].nil? or !params[:returnto].nil?
      redirect_to params[:returnto] || :back
    else # rucne vyplnene url, nejde se "vratit"
      if @reimported_osm_trail.nil?
        render text: "Import se nepodařil, relace neexistuje, zkuste to za chvíli znovu"
      else
        redirect_to osm_trail_url(@reimported_osm_trail)
      end
    end
  end

  def reimport_all
    @reversed_ids = OsmTrail.select{ |osm_trail| osm_trail.reversed?}.collect(&:osmid)
    
    url = "http://overpass-api.de/api/interpreter?data=%5Bout%3Ajson%5D%3Barea%5B%22wikidata%22%3D%22Q213%22%5D%3B%28relation%5B%22route%22%7E%22hiking%7Cfoot%22%5D%28area%29%3B%29%3B%3E%3E%3Bout%20body%3B%0A"
#     url = "http://overpass-api.de/api/interpreter?data=%5Bout%3Ajson%5D%3Barea%5B%22name%22%3D%22Vla%C5%A1im%22%5D%3B%28relation%5B%22route%22%7E%22hiking%7Cfoot%22%5D%28area%29%3B%29%3B%3E%3E%3Bout%20body%3B%0A"
#     url = "http://overpass-api.de/api/interpreter?data=%5Bout%3Ajson%5D%3Barea%5B%22name%22%3D%22%C4%8Cesko%22%5D%3B%28relation%5B%22route%22%7E%22hiking%7Cfoot%22%5D%28area%29%3B%29%3B%3E%3E%3Bout%20body%3B%0A"
#     render :text => url
    
    OsmTools.import_osm_data_from_json_string(open(url).read)
    
    @reversed_ids.each do |id|
      OsmTrail.find_by_osmid(id).reverse!
    end
    flash.notice = 'All trail successfully reiported from OSM'
    redirect_to :action=>:index
  end

  # DELETE /osm_trails/1
  # DELETE /osm_trails/1.json
  def destroy
    @osm_trail.destroy
    respond_to do |format|
      format.html { redirect_to osm_trails_url }
      format.json { head :no_content }
    end
  end

  def buglist
    if params[:all]
      @osm_trails = OsmTrail.all
    else
      @osm_trails = OsmTrail.joins(:note).where('osm_trail_notes.priority IS NOT NULL').order('coalesce(osm_trail_notes.priority, 999999),barva').page(params[:page]).select{|ot| ot.priority < 100}
    end
    respond_to do |format|
      format.xml { render :layout => false }
      format.html 
    end
  end

  private
    # Use callbacks to share common setup or constraints between actions.
    def set_osm_trail
      begin
        @osm_trail = (params[:id].match /osmid-(\d*)/) ? OsmTrail.find_by_osmid($1) : OsmTrail.find(params[:id])  # umožňuje použíte osmid-XXXX místo id v databázi
      rescue ActiveRecord::RecordNotFound
        flash.alert = "Tato OSM trasa není v databázi. Možná jde o starý odkaz a mezitím byla databáze reimportována."
        redirect_to :action=> :index
      end
    end

    # Never trust parameters from the scary internet, only allow the white list through.
    def osm_trail_params
      params[:osm_trail] || Hash.new()
    end
end
